@@ -230,3 +230,86 @@ class RecipeReader:
             "models": self.categories_to_models(categories),
             "values": np.array([1 if c in ingredients else 0 for c in columns]),
         }
+
+
+from gensim.models import Word2Vec
+from sentence_transformers import SentenceTransformer
+from annoy import AnnoyIndex
+import pandas as pd
+from ..data.embeddings import mean_ing_embedding
+from qdrant_client import QdrantClient
+
+
+ING_EMBEDDING_LEN = 128
+TAG_EMBEDDING_LEN = 384
+
+
+class TagReader:
+    def __init__(self):
+        tag_model_path = os.path.join(ROOT_DIR, "models", "tag_embedding.model")
+        self.tag_model = SentenceTransformer(tag_model_path)
+
+        ing_model_path = os.path.join(ROOT_DIR, "models", "ing_embedding.model")
+        self.ing_model = Word2Vec.load(ing_model_path)
+
+        embedding_csv_path = os.path.join(
+            ROOT_DIR, "data", "mean_ing_embedding_per_tag.csv"
+        )
+        self.__mean_ing_e_per_tag = pd.read_csv(embedding_csv_path, index_col=[0])
+        self.tags = list(self.__mean_ing_e_per_tag.columns)
+        self.__mean_ing_e = np.array(mean_ing_embedding)
+
+        self.__tags_index = AnnoyIndex(TAG_EMBEDDING_LEN, "angular")
+        embedded_tags = self.tag_model.encode(self.tags)
+
+        for id, tag_embedding in enumerate(list(embedded_tags)):
+            self.__tags_index.add_item(id, tag_embedding)
+
+        self.__tags_index.build(10)
+
+        self.qdrant = QdrantClient(host="qdrant.pocketsomm.dev", port=443, https=True)
+
+    def generate_ing_embedding(self, ing):
+        return self.ing_model.wv[ing] if ing in self.ing_model.wv else None
+
+    def generate_tag_embedding(self, tag):
+        return self.tag_model.encode(tag)
+
+    def get_ing_embedding_of_tag(self, tag):
+        return self.__mean_ing_e_per_tag[tag].values
+
+    def get_ing_embedding_by_tags(self, tags):
+        embeddings = []
+        for tag in tags:
+            # get recipe ids with tag
+            embedding = (
+                self.get_ing_embedding_of_tag(self.get_closest_tag(tag))
+                - self.__mean_ing_e
+            )
+            embeddings.append(embedding)
+
+        embedding_sum = sum(embeddings)
+        return np.add(embedding_sum, self.__mean_ing_e)
+
+    def get_closest_tag(self, tag):
+        index = self.__tags_index.get_nns_by_vector(
+            self.generate_tag_embedding(tag), 1
+        )[0]
+
+        return self.tags[index]
+
+    def read(self, tags):
+        embedding = self.get_ing_embedding_by_tags(tags)
+        search_result = self.qdrant.search(
+            collection_name="recipe-ing",
+            query_vector=("embedding", embedding.tolist()),
+            limit=1,
+            with_vectors=True,
+            with_payload=True,
+        )[0]
+
+        values = search_result.vector["oneHot"]
+        mask = values != 0
+        values[mask] = 1.0
+
+        return {"values": np.array(values), "model": search_result.payload["models"]}
